@@ -49,7 +49,6 @@
 
 package com.openexchange.util.activator;
 
-import javax.annotation.ParametersAreNonnullByDefault;
 import javax.annotation.concurrent.Immutable;
 import com.google.common.collect.ImmutableSet;
 import com.openexchange.util.activator.impl.MandatoryServiceDependency;
@@ -62,7 +61,6 @@ import com.openexchange.util.activator.impl.OptionalServiceDependency;
  * @author <a href="mailto:pascal.bleser@open-xchange.com">Pascal Bleser</a>
  * @since v1.0.1
  */
-@ParametersAreNonnullByDefault
 @Immutable
 public interface ServiceDependencies {
     ImmutableSet<MandatoryServiceDependency<?>> mandatoryServiceDependencies();
