@@ -49,10 +49,21 @@
 
 package com.openexchange.util.activator.impl;
 
+import static com.openexchange.util.custom.base.NullUtil.absent;
+import static com.openexchange.util.custom.base.NullUtil.build;
 import static com.openexchange.util.custom.base.NullUtil.className;
+import static com.openexchange.util.custom.base.NullUtil.emptyList;
+import static com.openexchange.util.custom.base.NullUtil.emptySet;
 import static com.openexchange.util.custom.base.NullUtil.f;
+import static com.openexchange.util.custom.base.NullUtil.immutableCopyOf;
+import static com.openexchange.util.custom.base.NullUtil.immutableSet;
+import static com.openexchange.util.custom.base.NullUtil.listBuilder;
 import static com.openexchange.util.custom.base.NullUtil.logger;
+import static com.openexchange.util.custom.base.NullUtil.mapBuilder;
 import static com.openexchange.util.custom.base.NullUtil.notNull;
+import static com.openexchange.util.custom.base.NullUtil.optional;
+import static com.openexchange.util.custom.base.NullUtil.setBuilder;
+import static java.util.stream.Collectors.joining;
 import java.io.Closeable;
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Constructor;
@@ -71,8 +82,10 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.Queue;
 import java.util.Set;
-import javax.annotation.Nullable;
-import javax.annotation.ParametersAreNonnullByDefault;
+import java.util.function.Function;
+import java.util.function.Predicate;
+import java.util.stream.Collectors;
+import java.util.stream.Stream;
 import javax.management.ObjectName;
 import javax.servlet.http.HttpServlet;
 import org.osgi.framework.BundleContext;
@@ -82,12 +95,8 @@ import org.osgi.framework.ServiceRegistration;
 import org.osgi.util.tracker.ServiceTracker;
 import org.osgi.util.tracker.ServiceTrackerCustomizer;
 import org.slf4j.Logger;
-import com.google.common.base.Function;
 import com.google.common.base.Joiner;
 import com.google.common.base.Optional;
-import com.google.common.base.Predicate;
-import com.google.common.base.Predicates;
-import com.google.common.collect.FluentIterable;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
@@ -95,6 +104,8 @@ import com.google.common.collect.Iterables;
 import com.google.common.collect.Lists;
 import com.openexchange.ajax.requesthandler.AJAXActionService;
 import com.openexchange.ajax.requesthandler.AJAXActionServiceFactory;
+import com.openexchange.annotation.NonNull;
+import com.openexchange.annotation.Nullable;
 import com.openexchange.config.ForcedReloadable;
 import com.openexchange.config.Reloadable;
 import com.openexchange.dispatcher.DispatcherPrefixService;
@@ -119,6 +130,7 @@ import com.openexchange.util.activator.ServiceRef;
 import com.openexchange.util.activator.ServiceRegistrationHandler;
 import com.openexchange.util.activator.ServiceWithProperties;
 import com.openexchange.util.activator.SetOfServices;
+import com.openexchange.util.custom.base.NullUtil;
 import com.openexchange.util.custom.collect.ImmutableDictionary;
 
 /**
@@ -127,7 +139,6 @@ import com.openexchange.util.custom.collect.ImmutableDictionary;
  * @author <a href="mailto:pascal.bleser@open-xchange.com">Pascal Bleser</a>
  * @since v7.8.2
  */
-@ParametersAreNonnullByDefault
 public enum RegistrationTools {
     ;
     
@@ -162,9 +173,9 @@ public enum RegistrationTools {
                 
                 LOG.error(f("failed to invoke service %s (%s) constructor \"%s\" with parameters [%s]: %s: %s",
                     serviceInstanceName,
-                    FluentIterable.from(serviceTypes).transform(TO_SIMPLE_CLASS_NAME).join(Joiner.on(", ")),
+                    serviceTypes.stream().map(TO_SIMPLE_CLASS_NAME).collect(Collectors.joining(", ")),
                     constructor.toGenericString(),
-                    FluentIterable.of(values.get()).transform(TO_CLASS_NAME).join(Joiner.on(", ")),
+                    Stream.of(values.get()).map(TO_CLASS_NAME).collect(Collectors.joining(", ")),
                     className(t), t.getMessage()), t);
                 if (t instanceof RegistrationException) {
                     throw (RegistrationException) t;
@@ -195,7 +206,7 @@ public enum RegistrationTools {
                 ExceptionUtils.handleThrowable(t);
                 LOG.error(String.format("failed to close %s (%s)",
                     serviceInstanceName,
-                    FluentIterable.from(serviceTypes).transform(TO_SIMPLE_CLASS_NAME).join(Joiner.on(", "))),
+                    serviceTypes.stream().map(TO_SIMPLE_CLASS_NAME).collect(Collectors.joining(", "))),
                     t);
             }
         } else {
@@ -212,7 +223,7 @@ public enum RegistrationTools {
                 ExceptionUtils.handleThrowable(t);
                 LOG.error(String.format("failed to shut down %s (%s)",
                     serviceInstanceName,
-                    FluentIterable.from(serviceTypes).transform(TO_SIMPLE_CLASS_NAME).join(Joiner.on(", "))),
+                    serviceTypes.stream().map(TO_SIMPLE_CLASS_NAME).collect(Collectors.joining(", "))),
                     t);
             }
         }        
@@ -268,15 +279,15 @@ public enum RegistrationTools {
     
     public static final <T> ImmutableList<Constructor<T>> serviceConstructors(final @Nullable Constructor<T>[] constructors) {
         if (constructors == null) {
-            return ImmutableList.of();
+            return emptyList();
         }
-        final ImmutableList.Builder<Constructor<T>> b = ImmutableList.builder();
+        final ImmutableList.Builder<Constructor<T>> b = listBuilder();
         for (final Constructor<T> ctor : constructors) {
             if (ctor.getAnnotation(Service.class) != null) {
                 b.add(ctor);
             }
         }
-        return b.build();
+        return build(b);
     }
 
     public static final Optional<Object[]> pickDependencies(final Method method,
@@ -321,7 +332,7 @@ public enum RegistrationTools {
                     final Class<?> serviceSetType = a.value();
                     final ServiceSet<?> serviceSet = serviceSetMap.get(serviceSetType);
                     if (serviceSet == null) {
-                        return Optional.absent();
+                        return absent();
                     } else {
                         values[i] = serviceSet;
                     }
@@ -332,7 +343,7 @@ public enum RegistrationTools {
                     final Class<?> serviceListingType = a.value();
                     final ServiceListing<?> serviceListing = serviceListingMap.get(serviceListingType);
                     if (serviceListing == null) {
-                        return Optional.absent();
+                        return absent();
                     } else {
                         values[i] = serviceListing;
                     }
@@ -351,7 +362,7 @@ public enum RegistrationTools {
                         }
                     }
                     if (! matched) {
-                        return Optional.absent();
+                        return absent();
                     }
                 }
             } else if (type.equals(Dictionary.class)) {
@@ -363,7 +374,7 @@ public enum RegistrationTools {
                     if (value != null && type.isInstance(value)) {
                         values[i] = Objects.toString(value);
                     } else {
-                        return Optional.absent();
+                        return absent();
                     }
                 } else {
                     boolean matched = false;
@@ -376,25 +387,20 @@ public enum RegistrationTools {
                         }
                     }
                     if (! matched) {
-                        return Optional.absent();
+                        return absent();
                     }
                 }
             }
         }
-        return Optional.of(values);
+        return optional(values);
     }
     
-    private static final Function<ServiceDependency<?>, String> TO_OSGI_FILTER = new Function<ServiceDependency<?>, String>() {
-        @Override public @Nullable String apply(@Nullable ServiceDependency<?> input) {
-            return input != null ? input.asFilter() : null;
-        }
-    };
-    
     public static final @Nullable Filter generateServiceFilter(final BundleContext context,
         final Collection<MandatoryServiceDependency<?>> mandatoryServiceDependencies,
         final Collection<OptionalServiceDependency<?>> optionalServiceDependencies) throws InvalidSyntaxException {
         
-        final List<ServiceDependencyTemplate<?>> dependencies = Lists.newArrayList(Iterables.concat(mandatoryServiceDependencies, optionalServiceDependencies));
+        @SuppressWarnings("null")
+        final @NonNull List<ServiceDependency<?>> dependencies = Lists.newArrayList(Iterables.concat(mandatoryServiceDependencies, optionalServiceDependencies));
         
         if (dependencies.isEmpty()) {
             return null;
@@ -402,12 +408,9 @@ public enum RegistrationTools {
         
         final String filter;
         if (dependencies.size() == 1) {
-            filter = TO_OSGI_FILTER.apply(dependencies.get(0));
+            filter = dependencies.get(0).asFilter();
         } else {
-            filter = String.format("(|%s)", FluentIterable.from(dependencies)
-                .transform(TO_OSGI_FILTER)
-                .join(Joiner.on(""))
-            );
+            filter = NullUtil.f("(|%s)", dependencies.stream().map(ServiceDependency::asFilter).collect(joining()));
         }
         return context.createFilter(filter);
     }
@@ -481,23 +484,26 @@ public enum RegistrationTools {
     }
     private static final Class<?>[] EMPTY_CLASS_ARRAY = new Class<?>[0];
     
-    public static final ImmutableSet<Class<?>> WELL_KNOWN_SERVICE_INTERFACES = ImmutableSet.<Class<?>>of(
+    @SuppressWarnings("null")
+    public static final @NonNull ImmutableSet<Class<?>> WELL_KNOWN_SERVICE_INTERFACES = ImmutableSet.<Class<?>>of(
         Reloadable.class
     );
-    public static final ImmutableSet<Class<?>> INTERFACES_TO_IGNORE_FOR_SERVICE_DISCOVERY = ImmutableSet.of(
+    @SuppressWarnings("null")
+    public static final @NonNull ImmutableSet<Class<?>> INTERFACES_TO_IGNORE_FOR_SERVICE_DISCOVERY = ImmutableSet.of(
         Reloadable.class, ForcedReloadable.class, Closeable.class
     );
-    public static final ImmutableSet<Class<?>> CLASSES_TO_IGNORE = ImmutableSet.of(
+    @SuppressWarnings("null")
+    public static final @NonNull ImmutableSet<Class<?>> CLASSES_TO_IGNORE = ImmutableSet.of(
         Enum.class, Object.class
     );
 
     public static <T> ImmutableSet<Class<?>> whatIsProvidedBy(final Class<T> serviceInstanceClass,
         final @Nullable Class<?> optServiceClass) throws RegistrationException {
         if (optServiceClass != null) {
-            return ImmutableSet.<Class<?>>of(optServiceClass);
+            return NullUtil.<Class<?>>immutableSet(optServiceClass);
         }
         
-        final Provides provides = serviceInstanceClass.getAnnotation(Provides.class);
+        final @Nullable Provides provides = serviceInstanceClass.getAnnotation(Provides.class);
         if (provides == null) {
             final ImmutableSet<Class<?>> interfaces;
             {
@@ -523,7 +529,7 @@ public enum RegistrationTools {
             if (interfaces.isEmpty()) {
                 // if it has a @Service annotation, than it is simply its own service class, so to speak
                 if (declaredAnnotation(Service.class, serviceInstanceClass) != null) {
-                    return ImmutableSet.<Class<?>>of(serviceInstanceClass);
+                    return NullUtil.<Class<?>>immutableSet(serviceInstanceClass);
                 } else {
                     // implements itself
                     // TODO it is not recommended to provide a class as an OSGi service facade, should we throw an exception
@@ -535,16 +541,17 @@ public enum RegistrationTools {
                     LOG.warn("{} registers itself as a class without interface, consider adding a @%s annotation to confirm that that is what you want to do",
                         serviceInstanceClass.getName(), Provides.class.getSimpleName());
                     // ... and then proceed ?
-                    return ImmutableSet.<Class<?>>of(serviceInstanceClass);
+                    return NullUtil.<Class<?>>immutableSet(serviceInstanceClass);
                 }
             } else if (interfaces.size() > 1) {
-                final Set<Class<?>> serviceInterfaces = FluentIterable.from(interfaces).filter(HAS_SERVICE_ANNOTATION).toSet();
+                final Set<Class<?>> serviceInterfaces = interfaces.stream().filter(HAS_SERVICE_ANNOTATION).collect(ImmutableSet.toImmutableSet());
                 if (serviceInterfaces.size() == 1) {
-                    return ImmutableSet.copyOf(serviceInterfaces);
+                    return immutableCopyOf(serviceInterfaces);
                 } else {
                     // filter out known interfaces to ignore
-                    final ImmutableSet<Class<?>> filteredInterfaces = 
-                        FluentIterable.from(interfaces).filter(Predicates.not(Predicates.in(INTERFACES_TO_IGNORE_FOR_SERVICE_DISCOVERY))).toSet();
+                    final ImmutableSet<Class<?>> filteredInterfaces = interfaces.stream()
+                        .filter(iface -> !INTERFACES_TO_IGNORE_FOR_SERVICE_DISCOVERY.contains(iface))
+                        .collect(ImmutableSet.toImmutableSet());
                     if (filteredInterfaces.isEmpty()) {
                         throw new RegistrationException(f("%s cannot be registered as it has no @%s annotation and implements several interfaces with @%s annotation: %s",
                             serviceInstanceClass.getName(), Provides.class.getSimpleName(), Service.class.getSimpleName(), Joiner.on(", ").join(serviceInterfaces)));
@@ -552,14 +559,18 @@ public enum RegistrationTools {
                         throw new RegistrationException(f("%s cannot be registered as it has no @%s annotation and implements several potential service interfaces (%s)",
                             serviceInstanceClass.getName(), Provides.class.getSimpleName(), Joiner.on(", ").join(filteredInterfaces)));
                     } else {
-                        return ImmutableSet.<Class<?>>of(Iterables.getOnlyElement(filteredInterfaces));
+                        @SuppressWarnings("null")
+                        final @NonNull Class<?> filteredInterface = Iterables.getOnlyElement(filteredInterfaces);
+                        return NullUtil.<Class<?>>immutableSet(filteredInterface);
                     }
                 }
             } else {
-                return ImmutableSet.<Class<?>>of(Iterables.getOnlyElement(interfaces));
+                @SuppressWarnings("null")
+                final @NonNull Class<?> iface = Iterables.getOnlyElement(interfaces);
+                return NullUtil.<Class<?>>immutableSet(iface);
             }
         }
-        final Class<?>[] serviceClasses = provides.value();
+        final @Nullable Class<?>[] serviceClasses = provides.value();
         if (serviceClasses == null || serviceClasses.length < 1) {
             throw new RegistrationException(f("%s cannot be registered because @%s is empty",
                 serviceInstanceClass.getName(), Provides.class.getSimpleName()));
@@ -579,12 +590,12 @@ public enum RegistrationTools {
             }
         }
         
-        return b.build();
+        return build(b);
     }
     
     public static final Predicate<Class<?>> HAS_SERVICE_ANNOTATION = new Predicate<Class<?>>() {
         @Override
-        public boolean apply(@Nullable Class<?> input) {
+        public boolean test(@Nullable Class<?> input) {
             return input != null && input.getAnnotation(Service.class) != null;
         }
     };
@@ -672,7 +683,7 @@ public enum RegistrationTools {
         final T serviceInstance,
         final Dictionary<String, ?> properties,
         final String path) {
-        final ImmutableSet<Class<?>> serviceClasses = ImmutableSet.<Class<?>>of(HttpServlet.class);
+        final ImmutableSet<Class<?>> serviceClasses = immutableSet(HttpServlet.class);
 
         final ImmutableSet<MandatoryServiceDependency<?>> mandatoryDependencies;
         final ImmutableSet<OptionalServiceDependency<?>> optionalDependencies;
@@ -751,8 +762,8 @@ public enum RegistrationTools {
         final MBeanRegistrar<S, T> r = new MBeanRegistrar<S, T>(bundleContext, registry, objectName, mbeanInterface, mbean);
         trackAndOpen(
             bundleContext,
-            ImmutableSet.<MandatoryServiceDependency<?>>of(new MandatoryServiceDependency<>(ManagementService.class)),
-            ImmutableSet.<OptionalServiceDependency<?>>of(),
+            immutableSet(new MandatoryServiceDependency<>(ManagementService.class)),
+            emptySet(),
             r,
             notNull(objectName.toString())
         );
@@ -769,13 +780,13 @@ public enum RegistrationTools {
         
         final ImmutableSet<Class<? extends AJAXActionService>> actionSet;
         {
-            final ImmutableSet.Builder<Class<? extends AJAXActionService>> b = ImmutableSet.builder();
+            final ImmutableSet.Builder<Class<? extends AJAXActionService>> b = setBuilder();
             for (final Class<? extends AJAXActionService> action : actions) {
                 if (action != null) {
                     b.add(action);
                 }
             }
-            actionSet = b.build();
+            actionSet = build(b);
         }
         
         registerServiceInstance(bundleContext, registry,
@@ -840,16 +851,15 @@ public enum RegistrationTools {
     }
     
     public static <S, T extends S> ServiceRegistration<S> registerIntoContext(final BundleContext bundleContext,
-        final Iterable<?> serviceClasses,
+        final Collection<?> serviceClasses,
         final T serviceInstance,
         final @Nullable Dictionary<String, ?> properties) {
         final ServiceRegistration<S> serviceRegistration;
         try {
-            final String[] serviceClassesAsArray = FluentIterable.from(serviceClasses)
-                .filter(Predicates.notNull())
-                .transform(CLASS_AWARE_TO_STRING)
-                .toArray(String.class)
-                ;
+            final String[] serviceClassesAsArray = serviceClasses.stream()
+                .filter(Objects::nonNull)
+                .map(CLASS_AWARE_TO_STRING)
+                .toArray(String[]::new);
             @SuppressWarnings("unchecked")
             final ServiceRegistration<S> r = (ServiceRegistration<S>) bundleContext.registerService(
                 serviceClassesAsArray,
@@ -884,14 +894,14 @@ public enum RegistrationTools {
     public static final ImmutableMap<Class<?>, ServiceSet<?>> serviceSets(final BundleContext bundleContext,
         final ActivatorRegistry registry,
         final ImmutableSet<Class<?>> serviceSetClasses) {
-        final ImmutableMap.Builder<Class<?>, ServiceSet<?>> b = ImmutableMap.builder();
+        final ImmutableMap.Builder<Class<?>, ServiceSet<?>> b = mapBuilder();
         for (final Class<?> serviceSetClass : serviceSetClasses) {
             if (serviceSetClass != null) {
                 final ServiceSet<?> ss = serviceSet(bundleContext, registry, serviceSetClass);
                 b.put(serviceSetClass, ss);
             }
         }
-        return b.build();
+        return build(b);
     }
 
     public static final <S> ServiceListing<S> serviceListing(final BundleContext bundleContext,
@@ -906,14 +916,14 @@ public enum RegistrationTools {
     public static final ImmutableMap<Class<?>, ServiceListing<?>> serviceListings(final BundleContext bundleContext,
         final ActivatorRegistry registry,
         final ImmutableSet<Class<?>> serviceListingClasses) {
-        final ImmutableMap.Builder<Class<?>, ServiceListing<?>> b = ImmutableMap.builder();
+        final ImmutableMap.Builder<Class<?>, ServiceListing<?>> b = mapBuilder();
         for (final Class<?> serviceListingClass : serviceListingClasses) {
             if (serviceListingClass != null) {
                 final ServiceListing<?> sl = serviceListing(bundleContext, registry, serviceListingClass);
                 b.put(serviceListingClass, sl);
             }
         }
-        return b.build();
+        return build(b);
     }
     
     public static final <S> void track(final BundleContext bundleContext,
@@ -986,16 +996,18 @@ public enum RegistrationTools {
         return serviceTracker;
     }
     
+    @SuppressWarnings("null")
     public static final ImmutableSet<Class<?>> KNOWN_INDIRECTIONS = ImmutableSet.<Class<?>>of(
         Remote.class
     );
-
+    
     public static final Function<Object, String> TO_CLASS_NAME = new Function<Object, String>() {
         @Override
         public @Nullable String apply(final @Nullable Object input) {
             return input != null ? className(input) : null;
         }
     };
+
     public static final Function<Map.Entry<Class<?>, Object>, String> ENTRY_TO_STRING = new Function<Map.Entry<Class<?>, Object>, String>() {
         @Override
         public @Nullable String apply(final @Nullable Map.Entry<Class<?>, Object> input) {
@@ -1022,9 +1034,10 @@ public enum RegistrationTools {
             return input != null ? input.getSimpleName() : null;
         }
     };
+    
     public static final Function<Object, String> CLASS_AWARE_TO_STRING = new Function<Object, String>() {
         @Override
-        public @Nullable String apply(final @Nullable Object input) {
+        public @Nullable String apply(@Nullable Object input) {
             if (input == null) {
                 return null;
             }
@@ -1033,7 +1046,7 @@ public enum RegistrationTools {
             }
             return input.toString();
         }
-    };    
+    };
 
     public static final String[] concatArray(final String arg0, final String... moreArgs) {
         final String[] all;
@@ -1070,7 +1083,7 @@ public enum RegistrationTools {
                 set.addAll(d.propertiesOfInterest());
             }
         }
-        return ImmutableSet.copyOf(set);
+        return NullUtil.immutableSetCopyOf(set);
     }
 
 }
