@@ -49,7 +49,15 @@
 
 package com.openexchange.util.activator.impl;
 
+import static com.google.common.collect.ImmutableList.toImmutableList;
+import static com.openexchange.util.custom.base.NullUtil.build;
+import static com.openexchange.util.custom.base.NullUtil.immutableSetCopyOf;
+import static com.openexchange.util.custom.base.NullUtil.listBuilder;
+import static com.openexchange.util.custom.base.NullUtil.logger;
+import static com.openexchange.util.custom.base.NullUtil.mapBuilder;
+import static java.util.stream.Collectors.joining;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Dictionary;
 import java.util.LinkedList;
 import java.util.List;
@@ -59,24 +67,23 @@ import java.util.Set;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.concurrent.locks.ReentrantLock;
-import javax.annotation.Nonnull;
-import javax.annotation.Nullable;
-import javax.annotation.ParametersAreNullableByDefault;
+
+
 import javax.annotation.concurrent.ThreadSafe;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.ServiceReference;
-import com.google.common.base.Joiner;
-import com.google.common.base.Predicates;
-import com.google.common.collect.FluentIterable;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
+import com.openexchange.annotation.NonNull;
+import com.openexchange.annotation.Nullable;
 import com.openexchange.config.Reloadable;
 import com.openexchange.osgi.DeferredActivator;
 import com.openexchange.osgi.ServiceListing;
 import com.openexchange.osgi.ServiceSet;
 import com.openexchange.osgi.console.ServiceStateLookup;
 import com.openexchange.util.activator.ServiceDependencyResolver;
+import com.openexchange.util.custom.base.NullUtil;
 import com.openexchange.util.custom.collect.ImmutableDictionary;
 
 /**
@@ -86,16 +93,15 @@ import com.openexchange.util.custom.collect.ImmutableDictionary;
  * @author <a href="mailto:pascal.bleser@open-xchange.com">Pascal Bleser</a>
  * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
  */
-@ParametersAreNullableByDefault
 @ThreadSafe
 public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extends S> implements ExtendedServiceTrackerCustomizer<S, T> {
 
-    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(ServiceAwareDependentServiceRegistererTemplate.class);
+    private static final org.slf4j.Logger LOG = logger(ServiceAwareDependentServiceRegistererTemplate.class);
     
     private static final class ResolvedService {
-        public @Nonnull final Object serviceObject; 
-        public @Nonnull final Dictionary<String, ?> properties;
-        public ResolvedService(@Nonnull Object serviceObject, @Nonnull Dictionary<String, ?> properties) {
+        public final Object serviceObject; 
+        public final Dictionary<String, ?> properties;
+        public ResolvedService(Object serviceObject, Dictionary<String, ?> properties) {
             this.serviceObject = serviceObject;
             this.properties = properties;
         }
@@ -104,7 +110,7 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
             return Objects.hash(serviceObject, properties);
         }
         @Override
-        public boolean equals(Object obj) {
+        public boolean equals(@Nullable Object obj) {
             if (obj == null) {
                 return false;
             }
@@ -118,44 +124,50 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
     }
     
     private final ReentrantLock lock = new ReentrantLock();
-    protected @Nonnull final BundleContext context;
-    protected @Nonnull final ImmutableSet<Class<?>> serviceTypes;
-    protected @Nonnull final String serviceInstanceName;
-    protected @Nonnull final Dictionary<String, ?> properties;
-    protected @Nonnull final ImmutableMap<Class<?>, ServiceSet<?>> serviceSets;
-    protected @Nonnull final ImmutableMap<Class<?>, ServiceListing<?>> serviceListings;
-    protected @Nonnull final ImmutableSet<String> propertiesOfInterest;
+    protected final BundleContext context;
+    protected final ImmutableSet<Class<?>> serviceTypes;
+    protected final String serviceInstanceName;
+    protected final Dictionary<String, ?> properties;
+    protected final ImmutableMap<Class<?>, ServiceSet<?>> serviceSets;
+    protected final ImmutableMap<Class<?>, ServiceListing<?>> serviceListings;
+    protected final ImmutableSet<String> propertiesOfInterest;
     private final ImmutableMap<MandatoryServiceDependency<?>, AtomicReference<ResolvedService>> mandatoryDependencyMap;
     private final ImmutableList<OptionalServiceDependency<?>> optionalDependencyList;
     private final AtomicBoolean registered = new AtomicBoolean(false);
 
-    public ServiceAwareDependentServiceRegistererTemplate(@Nonnull BundleContext context,
-        @Nonnull ImmutableSet<Class<?>> serviceTypes,
-        @Nonnull String serviceInstanceName,
-        @Nonnull Dictionary<String, ?> properties,
-        @Nonnull Iterable<MandatoryServiceDependency<?>> mandatoryServices,
-        @Nonnull Iterable<OptionalServiceDependency<?>> optionalServices,
-        @Nonnull ImmutableMap<Class<?>, ServiceSet<?>> serviceSets,
-        @Nonnull ImmutableMap<Class<?>, ServiceListing<?>> serviceListings,
-        @Nonnull Set<String> propertiesOfInterest) {
+    public ServiceAwareDependentServiceRegistererTemplate(
+        BundleContext context,
+        ImmutableSet<Class<?>> serviceTypes,
+        String serviceInstanceName,
+        Dictionary<String, ?> properties,
+        Collection<MandatoryServiceDependency<?>> mandatoryServices,
+        Collection<OptionalServiceDependency<?>> optionalServices,
+        ImmutableMap<Class<?>, ServiceSet<?>> serviceSets,
+        ImmutableMap<Class<?>, ServiceListing<?>> serviceListings,
+        Set<String> propertiesOfInterest) {
         super();
         this.context = context;
         this.serviceTypes = serviceTypes;
         this.serviceInstanceName = serviceInstanceName;
         this.properties = properties;
         {
-            final ImmutableMap.Builder<MandatoryServiceDependency<?>, AtomicReference<ResolvedService>> mandatory = ImmutableMap.builder();
+            final ImmutableMap.Builder<MandatoryServiceDependency<?>, AtomicReference<ResolvedService>> mandatory = mapBuilder();
             for (final MandatoryServiceDependency<?> neededService : mandatoryServices) {
                 if (neededService != null) {
                     mandatory.put((MandatoryServiceDependency<?>) neededService, new AtomicReference<ResolvedService>(null));
                 }
             }
-            this.mandatoryDependencyMap = mandatory.build();
-            this.optionalDependencyList = FluentIterable.from(optionalServices).filter(Predicates.notNull()).toList();
+            this.mandatoryDependencyMap = build(mandatory);
+
+            {
+                @SuppressWarnings("null")
+                final @NonNull ImmutableList<OptionalServiceDependency<?>> r = optionalServices.stream().filter(Objects::nonNull).collect(toImmutableList());
+                this.optionalDependencyList = r;
+            }
         }
         this.serviceSets = serviceSets;
         this.serviceListings = serviceListings;
-        this.propertiesOfInterest = ImmutableSet.copyOf(propertiesOfInterest);
+        this.propertiesOfInterest = immutableSetCopyOf(propertiesOfInterest);
         setState();
     }
     
@@ -176,7 +188,7 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
     protected void onOpenedExtension() {
     }
 
-    private static @Nonnull ImmutableDictionary<String, ?> snapshotProperties(final ServiceReference<?> reference, final Set<String> keys) {
+    private static ImmutableDictionary<String, ?> snapshotProperties(final ServiceReference<?> reference, final Set<String> keys) {
         final ImmutableMap.Builder<String, Object> b = ImmutableMap.builder();
         for (final String key : keys) {
             final Object value = reference.getProperty(key);
@@ -188,11 +200,11 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
     }
     
     @Override
-    public final T addingService(final ServiceReference<S> reference) {
+    public final T addingService(final @Nullable ServiceReference<S> reference) {
         if (reference == null) {
             throw new IllegalArgumentException("ServiceReference parameter is null");
         }
-        final @Nonnull T obj;
+        final T obj;
         {
             final S s = context.getService(reference);
             if (s == null) {
@@ -211,9 +223,19 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
                 if (dep != null && dep.matchesService(obj, reference)) {
                     final ResolvedService oldService = e.getValue().getAndSet(r);
                     needsRegistration &= (oldService == null);
+                    if (LOG.isTraceEnabled() && oldService == null) {
+                        LOG.trace("dependency resolution for {}: inbound service '{}' matches dependency '{}'", serviceInstanceName, obj.getClass().getName(), dep.identify());
+                    }
                 } else {
                     if (e.getValue().get() == null) {
                         unresolvedMandatoryServiceDependencies.add(e.getKey());
+                        if (LOG.isTraceEnabled()) {
+                            if (dep != null) {
+                                LOG.trace("dependency resolution for {}: still unresolved: '{}'", serviceInstanceName, dep.identify());
+                            } else {
+                                LOG.warn("dependency resolution for {}: service dependency is null", serviceInstanceName);
+                            }
+                        }
                     }
                 }
             }
@@ -222,8 +244,9 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
         }
         if (! unresolvedMandatoryServiceDependencies.isEmpty()) {
             if (LOG.isTraceEnabled()) {
-                LOG.trace("{} has {} unresolved service dependencies: {}", serviceInstanceName,
-                    unresolvedMandatoryServiceDependencies.size(), Joiner.on(", ").join(unresolvedMandatoryServiceDependencies));
+                LOG.trace("{} has {} unresolved mandatory service dependencies: {}", serviceInstanceName,
+                    unresolvedMandatoryServiceDependencies.size(),
+                    unresolvedMandatoryServiceDependencies.stream().map(d -> d.identify()).collect(joining(", ")));
             }
             needsRegistration = false;
         }
@@ -236,7 +259,7 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
         
         if (needsRegistration) {
             if (registered.compareAndSet(false, true)) {
-                LOG.trace("{} has its {} service(s) resolved, proceeding to registration",
+                LOG.trace("{} has its {} mandatory service dependencies resolved, proceeding to registration",
                     serviceInstanceName, mandatoryDependencyMap.size());
                 registerService(snapshotDependencies());
             }
@@ -246,8 +269,8 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
     }
     
     private final ServiceDependencyResolver snapshotDependencies() {
-        final ImmutableMap.Builder<MandatoryServiceDependency<?>, Object> m = ImmutableMap.builder();
-        final ImmutableList.Builder<OptionalServiceDependency<?>> o = ImmutableList.builder();
+        final ImmutableMap.Builder<MandatoryServiceDependency<?>, Object> m = mapBuilder();
+        final ImmutableList.Builder<OptionalServiceDependency<?>> o = listBuilder();
         lock.lock();
         try {
             for (final Map.Entry<MandatoryServiceDependency<?>, AtomicReference<ResolvedService>> e : mandatoryDependencyMap.entrySet()) {
@@ -267,7 +290,7 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
         } finally {
             lock.unlock();
         }
-        return new ServiceDependencyResolverImpl(m.build(), o.build(), serviceSets, serviceListings, properties);
+        return new ServiceDependencyResolverImpl(build(m), build(o), serviceSets, serviceListings, properties);
     }
     
     protected abstract @Nullable Object register(final ServiceDependencyResolver resolver);
@@ -279,7 +302,12 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
                     if (LOG.isTraceEnabled()) {
                         LOG.trace("'{}' is {}", serviceInstanceName, Reloadable.class.getSimpleName());
                     }
-                    RegistrationTools.registerIntoContext(context, ImmutableSet.of(Reloadable.class), serviceInstance, null);
+                    RegistrationTools.registerIntoContext(
+                        context,
+                        NullUtil.immutableSet(Reloadable.class),
+                        serviceInstance,
+                        null
+                    );
                 }
             }
         } finally {
@@ -288,12 +316,12 @@ public abstract class ServiceAwareDependentServiceRegistererTemplate<S, T extend
     
 
     @Override
-    public final void modifiedService(ServiceReference<S> reference, T service) {
+    public final void modifiedService(@Nullable ServiceReference<S> reference, @Nullable T service) {
         // Nothing to do.
     }
 
     @Override
-    public final void removedService(ServiceReference<S> reference, T service) {
+    public final void removedService(@Nullable ServiceReference<S> reference, @Nullable T service) {
         if (reference == null || service == null) {
             return;
         }
