@@ -49,6 +49,13 @@
 
 package com.openexchange.util.custom.config;
 
+import static com.openexchange.util.custom.base.NullUtil.absent;
+import static com.openexchange.util.custom.base.NullUtil.emptyList;
+import static com.openexchange.util.custom.base.NullUtil.emptySet;
+import static com.openexchange.util.custom.base.NullUtil.immutableCopyOf;
+import static com.openexchange.util.custom.base.NullUtil.immutableSetCopyOf;
+import static com.openexchange.util.custom.base.NullUtil.optional;
+import static com.openexchange.util.custom.base.NullUtil.optionalFromNullable;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.InputStream;
@@ -61,21 +68,20 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
+import java.util.function.Function;
 import java.util.regex.Pattern;
 import java.util.regex.PatternSyntaxException;
-import javax.annotation.CheckReturnValue;
-import javax.annotation.Nullable;
-import javax.annotation.ParametersAreNonnullByDefault;
 import javax.mail.internet.idn.IDNA;
 import com.google.common.base.CharMatcher;
 import com.google.common.base.Charsets;
-import com.google.common.base.Function;
 import com.google.common.base.Optional;
 import com.google.common.base.Splitter;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Iterators;
 import com.google.common.io.Closeables;
+import com.openexchange.annotation.NonNull;
+import com.openexchange.annotation.Nullable;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.configuration.ConfigurationExceptionCodes;
 import com.openexchange.exception.OXException;
@@ -88,7 +94,6 @@ import com.openexchange.util.custom.base.EmptyPropertiesSupplier;
  * @author <a href="mailto:pascal.bleser@open-xchange.com">Pascal Bleser</a>
  * @since v1.2.0
  */
-@ParametersAreNonnullByDefault
 public class ConfigReader {
     
     /**
@@ -276,6 +281,10 @@ public class ConfigReader {
         return new ConfigPropertyPrefixFunction(prefix);
     }
     
+    public final String prefix(final String name) {
+        return prefix(this.prefix, name);
+    }
+    
     @SuppressWarnings("null")
     public static final String prefix(final @Nullable String prefix, final String name) {
         if (prefix == null) {
@@ -315,7 +324,6 @@ public class ConfigReader {
     }
     
     public final class MandatoryValueReader {
-        @CheckReturnValue
         public String str(final String name) throws OXException {
             final String n = n(name);
             final ConfigurationService cs = configService;
@@ -333,7 +341,6 @@ public class ConfigReader {
             return str;
         }
         
-        @CheckReturnValue
         public String str(final String name, final String defaultValue) {
             final String n = n(name);
             final ConfigurationService cs = configService;
@@ -351,7 +358,6 @@ public class ConfigReader {
             return str;
         }
         
-        @CheckReturnValue
         public long longNum(final String name, final long defaultValue) throws OXException {
             final Optional<String> str = optional.str(name);
             if (! str.isPresent()) {
@@ -364,7 +370,6 @@ public class ConfigReader {
             }
         }
 
-        @CheckReturnValue
         public long longNum(final String name) throws OXException {
             final String str = mandatory.str(name);
             try {
@@ -374,7 +379,6 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public int intNum(final String name, final int defaultValue) throws OXException {
             final Optional<String> str = optional.str(name);
             if (! str.isPresent()) {
@@ -386,7 +390,7 @@ public class ConfigReader {
                 throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(String.format("%s is not a numeric value", n(name)));
             }
         }
-        @CheckReturnValue
+
         public int intNum(final String name) throws OXException {
             final String str = mandatory.str(name);
             try {
@@ -395,7 +399,7 @@ public class ConfigReader {
                 throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(String.format("%s is not a numeric value", n(name)));
             }
         }
-        @CheckReturnValue
+
         public int positiveIntNum(final String name) throws OXException {
             final int i = mandatory.intNum(name);
             if (i < 0) {
@@ -405,7 +409,6 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public boolean bool(final String name, final boolean defaultValue) throws OXException {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
@@ -421,7 +424,6 @@ public class ConfigReader {
             throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(String.format("%s is not a boolean (true/false)", n(name)));
         }
 
-        @CheckReturnValue
         public boolean bool(final String name) throws OXException {
             final String str = mandatory.str(name);
             if (str.equalsIgnoreCase("true")) {
@@ -433,7 +435,6 @@ public class ConfigReader {
             throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(String.format("%s is not a boolean (true/false)", n(name)));
         }
         
-        @CheckReturnValue
         public String hostname(final String name, final String defaultValue) {
             final String str = IDNA.toASCII(mandatory.str(name, defaultValue));
             if (str != null) {
@@ -442,7 +443,7 @@ public class ConfigReader {
                 return defaultValue;
             }
         }
-        @CheckReturnValue
+
         public String hostname(final String name) throws OXException {
             final String str = IDNA.toASCII(mandatory.str(name));
             if (str != null) {
@@ -452,7 +453,6 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public int port(final String name) throws OXException {
             final int value = mandatory.intNum(name);
             if (value <= 0 || value > 65535) {
@@ -461,7 +461,7 @@ public class ConfigReader {
                 return value;
             }
         }
-        @CheckReturnValue
+
         public int port(final String name, final int defaultValue) throws OXException {
             final Optional<Integer> o = optional.intNum(name);
             if (! o.isPresent()) {
@@ -475,21 +475,25 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public Charset charset(final String name, final Charset defaultValue) throws OXException {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
                 return defaultValue;
             } else {
                 try {
-                    return Optional.fromNullable(Charset.forName(o.get())).or(defaultValue);
+                    final Charset cs = Charset.forName(o.get());
+                    if (cs != null) {
+                        return cs;
+                    } else {
+                        return defaultValue;
+                    }
                 } catch (final Exception e) {
                     throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(e, String.format("%s does not contain a valid charset: %s",
                         n(name), e.getMessage()));
                 }
             }
         }
-        @CheckReturnValue
+
         public Charset charset(final String name) throws OXException {
             final String str = mandatory.str(name);
             final @Nullable Charset result;
@@ -506,37 +510,28 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public ImmutableList<String> strList(final String name, final CharMatcher split) throws OXException {
-            final String str = mandatory.str(name);
-            final ImmutableList<String> list = ImmutableList.copyOf(Splitter.on(split).omitEmptyStrings().split(str));
-            if (list.isEmpty()) {
-                throw ConfigurationExceptionCodes.PROPERTY_MISSING.create(n(name));
-            } else {
-                return list;
-            }
+            final String str = mandatory.str(name, "");
+            @SuppressWarnings("null")
+            final @NonNull Iterable<String> iter = Splitter.on(split).omitEmptyStrings().split(str);
+            return immutableCopyOf(iter);
         }
-        @CheckReturnValue
+
         public ImmutableList<String> strList(final String name) throws OXException {
             return mandatory.strList(name, DEFAULT_SPLIT_PATTERN);
         }
         
-        @CheckReturnValue
         public ImmutableSet<String> strSet(final String name, final CharMatcher split) throws OXException {
-            final String str = mandatory.str(name);
-            final ImmutableSet<String> set = ImmutableSet.copyOf(Splitter.on(split).omitEmptyStrings().split(str));
-            if (set.isEmpty()) {
-                throw ConfigurationExceptionCodes.PROPERTY_MISSING.create(n(name));
-            } else {
-                return set;
-            }
+            final String str = mandatory.str(name, "");
+            @SuppressWarnings("null")
+            final @NonNull Iterable<String> iter = Splitter.on(split).omitEmptyStrings().split(str);
+            return immutableSetCopyOf(iter);
         }
-        @CheckReturnValue
+
         public ImmutableSet<String> strSet(final String name) throws OXException {
             return mandatory.strSet(name, DEFAULT_SPLIT_PATTERN);
         }
 
-        @CheckReturnValue
         public URI uri(String name) throws OXException {
             final String str = mandatory.str(name);
             try {
@@ -546,7 +541,6 @@ public class ConfigReader {
             }
         }
 
-        @CheckReturnValue
         public URI uri(String name, URI defaultValue) throws OXException {
             final Optional<String> str = optional.str(name);
             if (str.isPresent()) {
@@ -560,7 +554,6 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public URI uri(String name, String defaultValue) throws OXException {
             final Optional<String> str = optional.str(name);
             if (str.isPresent()) {
@@ -578,7 +571,6 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public Pattern pattern(String name) throws OXException {
             final String str = mandatory.str(name);
             final Pattern result;
@@ -594,7 +586,6 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public Pattern pattern(String name, Pattern defaultValue) throws OXException {
             final Optional<String> str = optional.str(name);
             if (str.isPresent()) {
@@ -614,7 +605,6 @@ public class ConfigReader {
             }
         }
 
-        @CheckReturnValue
         public Pattern pattern(String name, int flags, Pattern defaultValue) throws OXException {
             final Optional<String> str = optional.str(name);
             if (str.isPresent()) {
@@ -634,7 +624,6 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public Pattern pattern(String name, String defaultValue) throws OXException {
             final Optional<String> str = optional.str(name);
             if (str.isPresent()) {
@@ -664,7 +653,6 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public Pattern pattern(String name, int flags, String defaultValue) throws OXException {
             final Optional<String> str = optional.str(name);
             if (str.isPresent()) {
@@ -694,7 +682,6 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public Pattern pattern(String name, int flags) throws OXException {
             final String str = mandatory.str(name);
             final Pattern result;
@@ -711,11 +698,11 @@ public class ConfigReader {
         }
         
         public final class MandatoryPropertiesLoader {
-            @CheckReturnValue
+            @SuppressWarnings("null")
             public Properties file(final String filename) throws OXException {
                 return file(filename, Charsets.ISO_8859_1); // ISO_8859_1 is the default for .properties file as of the Java specification
             }
-            @CheckReturnValue
+
             public Properties file(final String filename, final Charset charset) throws OXException {
                 File file = new File(filename);
                 if (! file.isAbsolute()) {
@@ -760,21 +747,23 @@ public class ConfigReader {
                 }
                 return result;
             }
-            @CheckReturnValue
+            
+            @SuppressWarnings("null")
             public Properties file(final String filename, final Map<String, ?> defaults) throws OXException {
                 return file(filename, Charsets.ISO_8859_1, defaults); // ISO_8859_1 is the default for .properties file as of the Java specification
             }
-            @CheckReturnValue
+            
             public Properties file(final String filename, final Charset charset, final Map<String, ?> defaults) throws OXException {
                 final Properties defaultsProps = new Properties();
                 defaultsProps.putAll(defaults);
                 return file(filename, charset, defaultsProps);
             }
-            @CheckReturnValue
+            
+            @SuppressWarnings("null")
             public Properties file(final String filename, final Properties defaults) throws OXException {
                 return file(filename, Charsets.ISO_8859_1, defaults); // ISO_8859_1 is the default for .properties file as of the Java specification
             }
-            @CheckReturnValue
+            
             public Properties file(final String filename, final Charset charset, final Properties defaults) throws OXException {
                 File file = new File(filename);
                 if (! file.isAbsolute()) {
@@ -827,50 +816,47 @@ public class ConfigReader {
     public final MandatoryValueReader m = mandatory;
 
     public final class OptionalValueReader {
-        @CheckReturnValue
         public Optional<String> str(final String name) {
             final String n = n(name);
             final ConfigurationService cs = configService;
             if (cs == null) {
-                return Optional.absent();
+                return absent();
             }
             String str = cs.getProperty(n);
             if (str == null) {
-                return Optional.absent();
+                return absent();
             }
             str = str.replaceAll("^\\s*", "").replaceAll("\\s*$", "");
             if (str.isEmpty()) {
-                return Optional.absent();
+                return absent();
             }
-            return Optional.of(str);
+            return optional(str);
         }
         
-        @CheckReturnValue
         public Optional<Long> longNum(final String name) throws OXException {
             final Optional<String> str = optional.str(name);
             if (! str.isPresent()) {
-                return Optional.absent();
+                return absent();
             }
             try {
-                return Optional.fromNullable(Long.parseLong(str.get()));
+                return optionalFromNullable(Long.parseLong(str.get()));
             } catch (final NumberFormatException e) {
                 throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(String.format("%s is not a numeric value", n(name)));
             }
         }
 
-        @CheckReturnValue
         public Optional<Integer> intNum(final String name) throws OXException {
             final Optional<String> str = optional.str(name);
             if (! str.isPresent()) {
-                return Optional.absent();
+                return absent();
             }
             try {
-                return Optional.fromNullable(Integer.parseInt(str.get()));
+                return optionalFromNullable(Integer.parseInt(str.get()));
             } catch (final NumberFormatException e) {
                 throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(String.format("%s is not a numeric value", n(name)));
             }
         }
-        @CheckReturnValue
+
         public Optional<Integer> positiveIntNum(final String name) throws OXException {
             final Optional<Integer> o = optional.intNum(name);
             if (o.isPresent()) {
@@ -882,32 +868,29 @@ public class ConfigReader {
             return o;
         }
         
-        @CheckReturnValue
         public Optional<Boolean> bool(final String name) throws OXException {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return Optional.absent();
+                return absent();
             }
             final String str = o.get();
             if (str.equalsIgnoreCase("true")) {
-                return Optional.of(true);
+                return optional(true);
             }
             if (str.equalsIgnoreCase("false")) {
-                return Optional.of(false);
+                return optional(false);
             }
             throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(String.format("%s is not a boolean (true/false)", n(name)));
         }
         
-        @CheckReturnValue
         public Optional<String> hostname(final String name) {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return Optional.absent();
+                return absent();
             }
-            return Optional.fromNullable(IDNA.toASCII(o.get()));
+            return optionalFromNullable(IDNA.toASCII(o.get()));
         }
         
-        @CheckReturnValue
         public Optional<Integer> port(final String name) throws OXException {
             final Optional<Integer> i = optional.intNum(name);
             if (! i.isPresent()) {
@@ -921,14 +904,13 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public Optional<Charset> charset(final String name) throws OXException {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return Optional.absent();
+                return absent();
             } else {
                 try {
-                    return Optional.fromNullable(Charset.forName(o.get()));
+                    return optionalFromNullable(Charset.forName(o.get()));
                 } catch (final Exception e) {
                     throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(e, String.format("%s does not contain a valid charset: %s",
                         n(name), e.getMessage()));
@@ -936,114 +918,122 @@ public class ConfigReader {
             }
         }
         
-        @CheckReturnValue
         public ImmutableList<String> strList(final String name, final CharMatcher split, final Iterator<String> defaultValues) {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return ImmutableList.<String>copyOf(defaultValues);
+                return immutableCopyOf(defaultValues);
             }
-            return ImmutableList.copyOf(Splitter.on(split).omitEmptyStrings().split(o.get()));
+            @SuppressWarnings("null")
+            final @NonNull Iterable<String> iter = Splitter.on(split).omitEmptyStrings().split(o.get());
+            return immutableCopyOf(iter);
         }
-        @CheckReturnValue
+
         public ImmutableList<String> strList(final String name, final Iterator<String> defaultValues) {
             return optional.strList(name, DEFAULT_SPLIT_PATTERN, defaultValues);
         }
-        @CheckReturnValue
+        
+        @SuppressWarnings("null")
         public ImmutableList<String> strList(final String name, final CharMatcher split, final String... defaultValues) {
             return optional.strList(name, split, Iterators.forArray(defaultValues));
         }
-        @CheckReturnValue
+        
+        @SuppressWarnings("null")
         public ImmutableList<String> strList(final String name, final String... defaultValues) {
             return optional.strList(name, DEFAULT_SPLIT_PATTERN, Iterators.forArray(defaultValues));
         }
         
-        @CheckReturnValue
         public ImmutableList<String> strList(final String name, final CharMatcher split) {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return ImmutableList.of();
+                return emptyList();
             }
-            return ImmutableList.copyOf(Splitter.on(split).omitEmptyStrings().split(o.get()));
+            @SuppressWarnings("null")
+            final @NonNull Iterable<String> iter = Splitter.on(split).omitEmptyStrings().split(o.get());
+            return immutableCopyOf(iter);
         }
-        @CheckReturnValue
+
         public ImmutableList<String> strList(final String name) {
             return optional.strList(name, DEFAULT_SPLIT_PATTERN);
         }
         
-        @CheckReturnValue
         public ImmutableList<String> strList(final String name, final CharMatcher split, final Iterable<String> defaultValues) {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return ImmutableList.<String>copyOf(defaultValues);
+                return immutableCopyOf(defaultValues);
             }
-            return ImmutableList.copyOf(Splitter.on(split).omitEmptyStrings().split(o.get()));
+            @SuppressWarnings("null")
+            final @NonNull Iterable<String> iter = Splitter.on(split).omitEmptyStrings().split(o.get());
+            return immutableCopyOf(iter);
         }
-        @CheckReturnValue
+
         public ImmutableList<String> strList(final String name, final Iterable<String> defaultValues) {
             return optional.strList(name, DEFAULT_SPLIT_PATTERN, defaultValues);
         }
         
-        @CheckReturnValue
         public ImmutableSet<String> strSet(final String name, final CharMatcher split, final Iterator<String> defaultValues) {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return ImmutableSet.<String>copyOf(defaultValues);
+                return immutableSetCopyOf(defaultValues);
             }
-            return ImmutableSet.copyOf(Splitter.on(split).omitEmptyStrings().split(o.get()));
+            @SuppressWarnings("null")
+            final @NonNull Iterable<String> iter = Splitter.on(split).omitEmptyStrings().split(o.get());
+            return immutableSetCopyOf(iter);
         }
-        @CheckReturnValue
+
         public ImmutableSet<String> strSet(final String name, final Iterator<String> defaultValues) {
             return optional.strSet(name, DEFAULT_SPLIT_PATTERN, defaultValues);
         }
-        @CheckReturnValue
+        
+        @SuppressWarnings("null")
         public ImmutableSet<String> strSet(final String name, final CharMatcher split, final String... defaultValues) {
             return optional.strSet(name, split, Iterators.forArray(defaultValues));
         }
-        @CheckReturnValue
+        
+        @SuppressWarnings("null")
         public ImmutableSet<String> strSet(final String name, final String... defaultValues) {
             return optional.strSet(name, DEFAULT_SPLIT_PATTERN, Iterators.forArray(defaultValues));
         }
         
-        @CheckReturnValue
         public ImmutableSet<String> strSet(final String name, final CharMatcher split) {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return ImmutableSet.of();
+                return emptySet();
             }
-            return ImmutableSet.copyOf(Splitter.on(split).omitEmptyStrings().split(o.get()));
+            @SuppressWarnings("null")
+            final @NonNull Iterable<String> iter = Splitter.on(split).omitEmptyStrings().split(o.get());
+            return immutableSetCopyOf(iter);
         }
-        @CheckReturnValue
+
         public ImmutableSet<String> strSet(final String name) {
             return optional.strSet(name, DEFAULT_SPLIT_PATTERN);
         }
         
-        @CheckReturnValue
         public ImmutableSet<String> strSet(final String name, final CharMatcher split, final Iterable<String> defaultValues) {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return ImmutableSet.<String>copyOf(defaultValues);
+                return immutableSetCopyOf(defaultValues);
             }
-            return ImmutableSet.copyOf(Splitter.on(split).omitEmptyStrings().split(o.get()));
+            @SuppressWarnings("null")
+            final @NonNull Iterable<String> iter = Splitter.on(split).omitEmptyStrings().split(o.get());
+            return immutableSetCopyOf(iter);
         }
-        @CheckReturnValue
+
         public ImmutableSet<String> strSet(final String name, final Iterable<String> defaultValues) {
             return optional.strSet(name, DEFAULT_SPLIT_PATTERN, defaultValues);
         }
         
-        @CheckReturnValue
         public Optional<URI> uri(String name) throws OXException {
             final Optional<String> o = optional.str(name);
             if (! o.isPresent()) {
-                return Optional.absent();
+                return absent();
             }
             try {
-                return Optional.of(new URI(o.get()));
+                return optional(new URI(o.get()));
             } catch (final URISyntaxException e) {
                 throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(e, String.format("the value of %s is not a valid URI", n(name)));
             }
         }
         
-        @CheckReturnValue
         public Optional<Pattern> pattern(String name) throws OXException {
             final Optional<String> str = optional.str(name);
             if (str.isPresent()) {
@@ -1054,16 +1044,15 @@ public class ConfigReader {
                     throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(e, String.format("the value of %s is not a valid regular expression", n(name)));
                 }
                 if (result != null) {
-                    return Optional.of(result);
+                    return optional(result);
                 } else {
                     throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(String.format("the value of %s is not a valid regular expression (null result)", n(name)));
                 }
             } else {
-                return Optional.absent();
+                return absent();
             }
         }
         
-        @CheckReturnValue
         public Optional<Pattern> pattern(String name, int flags) throws OXException {
             final Optional<String> str = optional.str(name);
             if (str.isPresent()) {
@@ -1074,36 +1063,36 @@ public class ConfigReader {
                     throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(e, String.format("the value of %s is not a valid regular expression", n(name)));
                 }
                 if (result != null) {
-                    return Optional.of(result);
+                    return optional(result);
                 } else {
                     throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(String.format("the value of %s is not a valid regular expression (null result)", n(name)));
                 }
             } else {
-                return Optional.absent();
+                return absent();
             }
         }
         
         public final class OptionalPropertiesLoader {
-            @CheckReturnValue
+            @SuppressWarnings("null")
             public Optional<Properties> file(final String filename) throws OXException {
                 return file(filename, Charsets.ISO_8859_1); // ISO_8859_1 is the default for .properties file as of the Java specification
             }
-            @CheckReturnValue
+
             public Optional<Properties> file(final String filename, final Charset charset) throws OXException {
                 File file = new File(filename);
                 if (! file.isAbsolute()) {
                     final ConfigurationService cs = configService;
                     if (cs == null) {
-                        return Optional.absent();
+                        return absent();
                     }
                     file = cs.getFileByName(filename);
                     trackFile(filename);
                     if (file == null) {
-                        return Optional.absent();
+                        return absent();
                     }
                 }
                 if (! file.exists()) {
-                    return Optional.absent();
+                    return absent();
                 }
                 final Properties result = new Properties();
                 {
@@ -1131,13 +1120,15 @@ public class ConfigReader {
                         }
                     }
                 }
-                return Optional.of(result);
+                return optional(result);
             }
-            @CheckReturnValue
+            
+            @SuppressWarnings("null")
             public Properties fileOrEmpty(final String filename) throws OXException {
                 return file(filename).or(EmptyPropertiesSupplier.INSTANCE);
             }
-            @CheckReturnValue
+            
+            @SuppressWarnings("null")
             public Properties fileOrEmpty(final String filename, final Charset charset) throws OXException {
                 return file(filename, charset).or(EmptyPropertiesSupplier.INSTANCE);
             }
@@ -1147,6 +1138,7 @@ public class ConfigReader {
     public final OptionalValueReader optional = new OptionalValueReader();
     public final OptionalValueReader o = optional;
 
+    @SuppressWarnings("null")
     public static final CharMatcher DEFAULT_SPLIT_PATTERN = CharMatcher.anyOf("\r\n;, ");
 
 }
